<?php

$remote_address = getenv("REMOTE_ADDR");
$cl_id = get_client_id($remote_address);

$cr_id = 0;
$error_code = '';
$header_list = array();
$cr_content = '';
$sem_id = 0;

#### are we getting called by an embedded ad serve?
if(($cr_id = $c)
   || (select_creative($s, $w, $h)
       && record_delivery($cr_id, $remote_address, $cl_id, $s, $HTTP_USER_AGENT)))
{
  if($c)
  {
    retrieve_creative($cr_id);
    deliver_creative();
  }
  else
  {
    @sem_release($sem_id);
    retrieve_creative($cr_id);
    record_cookie($cl_id, $remote_address, $s, $w, $h, $cr_id);
    deliver_creative();
  }
}
else
{
  #### release the semaphore that was obtained in select_creative()
  @sem_release($sem_id);
  if($s == 0) $s = 0;

  deliver_default();
  log_msg(1, $cl_id, $remote_address, $s, $cr_id, 'ierr', $error_code, $HTTP_USER_AGENT);
}



####---------------------------------------------------------------------------
#### look for a cookie; if found, return its ID number.  Otherwise, generate
#### a new ID.
####---------------------------------------------------------------------------
function get_client_id($remote_address)
{
  global $HTTP_COOKIE_VARS;

  #### cookie is named OASISID
  if($id = $HTTP_COOKIE_VARS["OASISID"])
  {
    #### some poorly-coded browsers can barf up more than just our OASISID
    #### cookie (I've even seen some dump what looks like their entire
    #### cookie table!).  When this gets logged, it will wreak havoc on
    #### the log processor.  So just to be sure, we trim off everything past
    #### the 13-character unique ID.
    $id = substr($id, 0, 13);
  }
  else
  {
    $id = uniqid('');

    #### 5-year expiration on cookie
    setcookie("OASISID", $id, time() + 157680000, "/", ".[oasis_domain]");
  }

  return $id;
}



####---------------------------------------------------------------------------
#### given a section ID and the desired width and height, select a creative
#### from the pool of available creatives
####---------------------------------------------------------------------------
function select_creative($section, $width, $height)
{
  global $cr_id, $sem_id, $error_code;

  #### grab a semaphore; we don't want to enter these shared memory
  #### segments when another instance of this script is in there
  #### or if they're being reloaded by the maintenance scripts
  #### identifier:           O A 0 0
  if(!((@$sem_id = sem_get(0x4f410000, 1)) && @sem_acquire($sem_id)))
  {
    $error_code = 'no_sem';
    return 0;
  }

  #### attach to Hourly Assignments shared memory segment
  #### identifier:     O A S 0
  @$smh = shm_attach(0x4f415300, 0); 
  @$section_creatives = shm_get_var($smh, $section);
  @shm_detach($smh);

  if(!$section_creatives)
  {
    $error_code = 'no_sec_cr';
    return 0;
  }

  $secdim_creatives = $section_creatives[$width . "x" . $height];

  if(!$secdim_creatives)
  {
    #### big problem -- no creatives scheduled for this hour for this section
    $error_code = 'no_secdim_cr';
    return 0;
  }

  #### attach to Hourly Targets shared memory segment
  #### identifier:     O A S 1
  if(!(@$smh = shm_attach(0x4f415301, 0)))
  {
    $error_code = "no_ht_shm ($php_errormsg)";
    return 0;
  }

  #### loop through creatives, looking for best candidate
  $cr_id = 0;
  $total_weight = 0;
  $total_remaining = 0;
  foreach ($secdim_creatives as $c)
  {
    @$target_info = shm_get_var($smh, $c);

    #### we really don't want/expect this to happen, but we also don't
    #### want to bring delivery to a halt just because one creative's
    #### target info isn't retrieved from shared memory
    if(!$target_info) continue;

    list($target, $weight, $remaining, $clicks, $mtype) = $target_info;

    if($mtype != 'Image') continue;

    #print "creative $c: [$target, $weight, $remaining]<BR>\n";
    if($remaining > 0)
    {
      $total_remaining += $remaining;
      $remaining_ads[$c] = $remaining;
    }
    elseif($target == 0)
    {
      $total_weight += $weight;
      $weight_ads[$c] = $weight;
    }
  }
  @shm_detach($smh);

  #### should we select a 'remaining' ad or a 'weight' ad?
  if($total_remaining <= 0)
  {
    $rmax = $total_weight;
    $candidates = $weight_ads;
  }
  else
  {
    $rmax = $total_remaining;
    $candidates = $remaining_ads;
  }

  if($rmax > 0)
  {
    mt_srand((double)microtime()*1000000);
    $rand_num = mt_rand(0, $rmax);
    while(list($c, $w) = each($candidates))
    {
      $rand_num -= $w;
      if($rand_num <= 0)
      {
        $cr_id = $c;
        break;
      }
    }
  }
  else
  {
    #### the creatives assigned to this section are maxed out (all have
    #### targets, and all have remaining = 0)
    $error_code = 'secdim_cr_maxed_out';
    return 0;
  }

  return 1;
}


####---------------------------------------------------------------------------
####---------------------------------------------------------------------------
function record_delivery($cr_id, $ra, $cl_id, $s, $ua)
{
  global $error_code; 

  #### decrement the "remaining" field; we have to do this here so that
  #### the selection and decrementing is atomic (we're still inside the
  #### protection of the semaphore).  Of course, we'd prefer to do it
  #### after we've actually sent the content of the creative, but we're
  #### going to send the creative immediately after this step.  This
  #### is as close to the actual delivery as we can get; if we try to
  #### hold the semaphore until after the delivery is complete, we'll
  #### create an enormous bottleneck as we wait for slow connections.

  #### attach to Hourly Targets shared memory segment
  #### identifier:     O A S 1
  if(!(@$smh = shm_attach(0x4f415301, 0)))
  {
    $error_code = "no_ht_shm_2 ($php_errormsg)";
    return 0;
  }

  #### retrieve the current target info
  if(!@$target_info = shm_get_var($smh, $cr_id))
  {
    $error_code = "no_target_info ($php_errormsg)";
    return 0;
  }

  #### decrement "remaining" field and put back into SHM
  $target_info[2]--;
  if(@shm_put_var($smh, $cr_id, $target_info))
  {
    #### we log the delivery immediately so that it occurs right after
    #### the shared memory table is updated, reducing the chance that
    #### something might go wrong and the impression doesn't get logged.
    log_msg(0, $cl_id, $ra, $s, $cr_id, 'imp', '', $ua);
  }
  else
  {
    $error_code = "no_target_info_put ($php_errormsg)";
    return 0;
  }

  return 1;
}



####---------------------------------------------------------------------------
#### retrieve the content of the specified creative
####---------------------------------------------------------------------------
function retrieve_creative($cr_id)
{
  global $cr_content, $header_list;
  global $sem_id, $error_code;

  $error_code = '';

  #### attach to a shared memory segment
  #### identifier:     O A S 2
  @$smh = shm_attach(0x4f415302, 0); 
  @$creative_info = shm_get_var($smh, $cr_id);
  @shm_detach($smh);

  if(!$creative_info)
  {
    $error_code = "no_cr_content ($php_errormsg)";
    return 0;
  }

  list($content, $redirect, $mime_type, $animated) = $creative_info;

  if($redirect == 'Y')
  {
    mt_srand((double)microtime()*1000000);
    $random_number = mt_rand();
    $content = preg_replace("#\[CB\]#", $random_number, $content);
    array_push($header_list, "Location: $content");

    #### these seem to cause the same headaches that they do for locally
    #### served animated banners!
    #header("Pragma: no-cache");
    #header("Cache-Control: no-cache");
  }
  else
  {
    array_push($header_list, "Content-type: $mime_type");
    array_push($header_list,
               "Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT"); 

    #### set some headers to control caching; note that due to browser
    #### wackiness, we can't set these for animated GIFs
    if($animated == 'N')
    {
      array_push($header_list, "Expires: Sat, 13 Jun 1992 00:00:00 GMT");
      array_push($header_list, "Cache-Control: no-cache, must-revalidate");
      array_push($header_list, "Pragma: no-cache");
    }

    $cr_content = $content;
  }

  return 1;
}


####---------------------------------------------------------------------------
#### print out the headers and the content
####---------------------------------------------------------------------------
function deliver_creative()
{
  global $cr_content, $header_list;

  foreach($header_list as $h) header($h);
  print $cr_content;
}


####---------------------------------------------------------------------------
#### spit out a blank 1x1 GIF
####---------------------------------------------------------------------------
function deliver_default()
{
  header("Content-type: image/gif");
  header("Expires: Sat, 13 Jun 1992 00:00:00 GMT");
  header("Cache-Control: no-cache, must-revalidate");
  header("Pragma: no-cache");

  $content = sprintf("GIF89a%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c",
                                      0x01, 0x00,
  0x01, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x21, 0xf9, 0x04, 0x01, 0x00,
  0x00, 0x00, 0x00, 0x2c, 0x00, 0x00, 0x00, 0x00,
  0x01, 0x00, 0x01, 0x00, 0x40, 0x02, 0x02, 0x84,
  0x51, 0x00, 0x3b, 0x00);

  print $content;
}


####---------------------------------------------------------------------------
#### record which creative was delivered to this user so that if he clicks
#### through, we can send him to the right destination
####---------------------------------------------------------------------------
function record_cookie($client, $ra, $sec, $w, $h, $creative)
{
  #print "client: $client<BR>\n";

  $key_end = sprintf("%04x%04x%04x", $sec, $w, $h);

  #### save the creative twice, once with the client ID and once
  #### with the IP address (in case client is not accepting cookies,
  #### we'll have a fallback if he clicks through)
  $bucket_key1 = $client . $key_end;
  preg_match('#(\d+)\.(\d+)\.(\d+)\.(\d+)#', $ra, $ip);
  $bucket_key2 = sprintf("%02x%02x%02x%02x", $ip[1], $ip[2], $ip[3], $ip[4])
                 . $key_end;

  #### suppress warnings so that if database is temporarily down, we
  #### can at least deliver the creative
  @mysql_pconnect("[db_host]", "[db_user]", "[db_passwd]");
  @mysql_select_db("oasis");
  @mysql_query("replace into Delivery values ('$bucket_key1', $creative, NULL), ('$bucket_key2', $creative, NULL)");
}


####---------------------------------------------------------------------------
####---------------------------------------------------------------------------
function log_msg($get_sem, $cl_id, $ra, $section, $cr_id, $event, $code, $ua)
{
  $log_message = time()
     . "\t$ra\t$cl_id\t$event\t$section\t$cr_id\t$code\t$ua\n";

  #### grab a semaphore; we don't want to write to this log
  #### when another instance of this script is in there
  #### identifier:           O A O 0
  if($get_sem)
  {
    if(!((@$sem_id = sem_get(0x4f410000, 1)) && @sem_acquire($sem_id)))
      return 0;
  }

  error_log($log_message, 3, "[log_dir]/oasis.log");

  if($get_sem) @sem_release($sem_id);
}


?>
